#[macro_use]
extern crate rocket;

use rocket::{fs::FileServer};
use rocket::serde::json::Json;
use rocket::http::Status;
use rocket::Request;
use rocket::response::status::Custom;

use sqlx::postgres::PgPoolOptions;

use std::env;
use dotenv::dotenv;

mod auth;
mod encryption;
mod file;
mod user;
mod admin;
mod models;

use crate::file::*;
use crate::user::*;
use crate::admin::*;

#[catch(404)]
fn not_found(_: &Request) -> Custom<Json<&'static str>> {
    Custom(Status::NotFound, Json("Not Found"))
}

#[rocket::main]
async fn main() -> Result<(), rocket::Error> {
    dotenv().ok();

    let db_url = env::var("DATABASE_URL").expect("DATABASE_URL not set");
    let pool = PgPoolOptions::new()
        .max_connections(5)
        .connect(&db_url)
        .await
        .expect("Failed to connect to database");

    let figment = rocket::Config::figment()
        .merge(("port", 8082))
        .merge(("address", "0.0.0.0"));

    rocket::custom(figment)
        .manage(pool)
        .mount("/api", routes![
            register,
            login,
            get_user_info,
            delete_user,
            upload_file,
            download_file,
            download_shared,
            list_user_files,
            delete_file,
            share_file,
            update_role,
            update_quota,
        ])
        //.mount("/", FileServer::from("/app/static").rank(10))
        //.register("/", catchers![not_found])
        .launch()
        .await?;

    Ok(())
}
